
--[[ Configuration functions ]]--

local importFormats = {
	"AskMrRobot", "Pawn"
}

local ItemScoreConfig = {}

function ItemScore:InitOptions()
	
	ItemScoreConfig.selectedSet = self:GetDefaultSet()
	
	local importOptions = {
		desc = "Import",
		name = "Import",
		type = "group",
		handler = ItemScoreConfig,
		order = 20,
		
		args = {
			format = {
				type = "select",
				style = "dropdown",
				name = "Format",
				order = 10,
				get = "GetImportFormat",
				set = "SetImportFormat",
				values = importFormats,
			},
			name = {
				type = "input",
				name = "Name",
				order = 20,
				get = "GetImportName",
				set = "SetImportName",
			},
			codeHeader = {
				type = "header",
				name = "Code",
				order = 25,
			},
			code = {
				type = "input",
				name = "Code",
				width = "full",
				order = 30,
				multiline = true,
				get = "GetImportCode",
				set = "SetImportCode",
			},
			import = {
				type = "execute",
				name = "Import",
				func = "ImportScoreSet",
			},
		},
	}
	
	local exportOptions = {
		desc = "Export",
		name = "Export",
		type = "group",
		handler = ItemScoreConfig,
		order = 30,
		
		args = {
			format = {
				type = "select",
				style = "dropdown",
				name = "Format",
				order = 10,
				get = "GetExportFormat",
				set = "SetExportFormat",
				values = importFormats,
			},
			scoreSets = {
				type = "select",
				style = "dropdown",
				name = "Export ItemScore set",
				order = 20,
				values = {},
				get = "GetExportSet",
				set = "SetExportSet",
			},
			codeHeader = {
				type = "header",
				name = "Code",
				order = 25,
			},
			code = {
				type = "input",
				name = "Code",
				width = "full",
				order = 30,
				multiline = true,
				get = "GetExportCode",
			},
		},
	}
	
	local scoreSetOptions = {
		desc = "Score Sets",
		name = "Score Sets",
		type = "group",
		order = 10,
		handler = ItemScoreConfig,
		
		args = {
			scoreSets = {
				type = "select",
				style = "dropdown",
				name = "ItemScore set",
				order = 10,
				values = {},
				get = "GetSelectedSet",
				set = "SetSelectedSet",
			},
			operationHeader = {
				type = "header",
				name = "Score Set Operations",
				order = 15,
			},
			newName = {
			    type = "input",
			    name = "Name",
			    order = 20,
			    get = "GetNewName",
			    set = "SetNewName",
			},
			newSet = {
			    type = "execute",
			    name = "New",
			    order = 30,
			    func = "CreateNewSet",
			    width = "half",
			},
			copySet = {
			    type = "execute",
			    name = "Copy",
			    order = 40,
			    func = "CopySet",
			    width = "half",
			},
			renameSet = {
				type = "execute",
			    name = "Rename",
			    order = 50,
			    func = "RenameSet",
			    width = "half",
			},
			deleteSet = {
			    type = "execute",
			    name = "Delete",
			    order = 60,
			    func = "DeleteSet",
			    width = "half",
			},
			detailsHeader = {
				type = "header",
				name = "Score Set Options",
				order = 65,
			},
			setEnabled = {
				type = "toggle",
				name = "Display",
				order = 70,
				get = "GetEnabled",
				set = "SetEnabled",
				width = "half",
			},
			setParent = {
				type = "select",
				style = "dropdown",
				name = "Parent set",
				order = 75,
				values = {},
				get = "GetParent",
				set = "SetParent",
			},
			setPrecision = {
				type = "input",
			    name = "Precision",
			    order = 80,
			    get = "GetPrecision",
			    set = "SetPrecision",
			    width = "half",
			},
			setColour = {
				type = "color",
				name = "Colour",
				order = 90,
				get = "GetColour",
				set = "SetColour",
				width = "half",
			},
			setPattern = {
				type = "input",
			    name = "Display text",
			    desc = "%n - name, %a - score, %d - difference, %p - percent difference",
			    order = 100,
			    get = "GetPattern",
			    set = "SetPattern",
			},
			exampleText = {
				type = "description",
				name = "",
				order = 110,
			},
			statTable = {
				name = "Stat scores",
				type = "group",
				order = 120,
				args = {},
				width = "half",
			},
			availableStats = {
				name = "Available stats",
				type = "group",
				order = 130,
				args = {},
				width = "half",
			}
		},
	}
	
	self.options = {
		desc = "ItemScore",
		type = "group",
		childGroups = "tab",
		order = 10,
		handler = ItemScoreConfig,
		args = {importOptions = importOptions, scoreSetOptions = scoreSetOptions, exportOptions = exportOptions}
	}
	
	ItemScoreConfig.scoreSetOptionsArgs = scoreSetOptions.args
	ItemScoreConfig.scoreSetValues = ItemScoreConfig.scoreSetOptionsArgs.scoreSets.values
	ItemScoreConfig.setParentValues = ItemScoreConfig.scoreSetOptionsArgs.setParent.values
	ItemScoreConfig.exportValues = exportOptions.args.scoreSets.values
	ItemScoreConfig.exampleText = ItemScoreConfig.scoreSetOptionsArgs.exampleText
	
	ItemScoreConfig:UpdateStatTable()
	ItemScoreConfig:UpdateSetLists()
end

function ItemScore:RegisterSlashCommands()
	self:RegisterChatCommand('itemscore', 'OnCmd')
end

function ItemScoreConfig:UpdateStatTable()
	optionList = self.scoreSetOptionsArgs
	local scoreSet = ItemScore:GetScoreSet(self.selectedSet)

	-- if nothing selected, clear out the table
	if not scoreSet then
		for i in pairs(optionList.statTable.args) do
			if optionList.statTable.args[i].type == "group" then
				optionList.statTable.args[i] = nil
			end
		end
		for i in pairs(optionList.availableStats.args) do
			if optionList.availableStats.args[i].type == "group" then
				optionList.availableStats.args[i] = nil
			end
		end
	else
		for i in pairs(scoreSet.info) do
			local capVal = ""
			
			if scoreSet.info[i].cap and scoreSet.info[i].cap ~= -1 then
				capVal = " (<=" .. scoreSet.info[i].cap .. ")"
			end
			
			if not optionList.statTable.args[i] then
			 
				local options = {
					score = {
						name = "Score",
						type = "input",
						order = 110,
						set = "SetScore",
						get = "GetScore",
						arg = i					
					},
					cap = {
						name = "Cap",
						type = "input",
						order = 200,
						set = "SetCap",
						get = "GetCap",
						arg = i
					},
					remove = {
						type = "execute",
					    name = "Remove",
					    order = 210,
					    func = "RemoveStat",
					    arg = i,
					}
				}
							
				-- quietly remove stat if its not supported
				if not ItemScore.supportedStats[i] then
					scoreSet.info[i] = nil
				else
					optionList.statTable.args[i] = {
						name = ItemScore.supportedStats[i] .. " - " .. scoreSet.info[i].score .. capVal,
						type = "group",
						width = "half",
						args = options
					}
				end
			else
				-- just update the name
				optionList.statTable.args[i].name = ItemScore.supportedStats[i] .. " - " .. scoreSet.info[i].score .. capVal
			end
		end
		
		-- remove stats that are no longer in the list
		for i in pairs(optionList.statTable.args) do
			if not scoreSet.info[i] then
				optionList.statTable.args[i] = nil
			end
		end
		
		-- now populate another list with stats that aren't in the set
		for i in pairs(ItemScore.supportedStats) do
			if not scoreSet.info[i] then
				local options = {
					add = {
						type = "execute",
					    name = "Add",
					    order = 210,
					    func = "AddStat",
					    arg = i,
					}
				}
						
				optionList.availableStats.args[i] = {
					name = ItemScore.supportedStats[i],
					type = "group",
					width = "half",
					args = options
				}
			end
		end
		
		-- remove stats that are in the set
		for i in pairs(optionList.availableStats.args) do
			if scoreSet.info[i] then
				optionList.availableStats.args[i] = nil
			end
		end
	end
end

function ItemScoreConfig:UpdateSetLists()
	ItemScoreConfig:UpdateSetList(self.scoreSetValues, nil, false)
	ItemScoreConfig:UpdateSetList(self.setParentValues, self.selectedSet, true)
	ItemScoreConfig:UpdateSetList(self.exportValues, nil, false)
end

function ItemScoreConfig:UpdateSetList(optionList, ignoreSet, addEmpty)
	local setList = ItemScore:GetScoreSets()
	
	-- add new sets that aren't in optionList
	for i in pairs(setList) do
		if not optionList[i] then
			optionList[i] = i
		end
	end
	
	-- remove from optionList spells that aren't in spellList
	for i in pairs(optionList) do
		if not setList[i] then
			optionList[i] = nil
		end
	end
	
	if ignoreSet then
	    optionList[ignoreSet] = nil
	end
	
	self:UpdateExampleText(self.exampleText, self.selectedSet)
	
	if addEmpty then
	    optionList[""] = ""
	end
end

function ItemScoreConfig:UpdateExampleText()
	if not self.selectedSet then
		self.exampleText.name = ""
		return
	end
	
	local score = 123.456789
	local equippedScore = 125.567891
	
	self.exampleText.name = ItemScore:MakeTooltipText(ItemScore:GetScoreSet(self.selectedSet), score, score, equippedScore, equippedScore)
end

-- Handlers

function ItemScoreConfig:GetSelectedSet(info, value)
	-- return the selected set name
	return self.selectedSet
end

function ItemScoreConfig:SetSelectedSet(info, value)
	self.selectedSet = value
	
	self:UpdateStatTable()
	self:UpdateExampleText()
	self:UpdateSetLists()
end

function ItemScoreConfig:GetNewName(info, value)
	return self.newName
end

function ItemScoreConfig:SetNewName(info, value)
	self.newName = value
end

function ItemScoreConfig:CreateNewSet()
	if not self.newName or self.newName == '' then
		-- display error here
	else
		ItemScore:CreateNewSet(self.newName)
		self:SetSelectedSet(nil, self.newName)
		self:UpdateSetLists()
	end
end

function ItemScoreConfig:CopySet()
	if not self.newName or self.newName == '' then
		-- display error here
	elseif not self.selectedSet then
		-- display error here
	else
		ItemScore:CopySet(self.selectedSet, self.newName)
		self:SetSelectedSet(nil, self.newName)
		self:UpdateSetLists()
	end 
end

function ItemScoreConfig:RenameSet()
	if not self.newName or self.newName == '' then
		-- display error here
	elseif not self.selectedSet then
		-- display error here
	else
		ItemScore:RenameSet(self.selectedSet, self.newName)
		
		self.selectedSet = self.newName
		self:UpdateSetLists()
	end 
end

function ItemScoreConfig:DeleteSet()
	if not self.selectedSet then
		-- display error here
	else
		ItemScore:DeleteSet(self.selectedSet)
		
		self:SetSelectedSet()
		self:UpdateSetLists()
	end 
end

function ItemScoreConfig:SetScore(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set value with no selected set")
	else
		ItemScore:SetScore(self.selectedSet, info.arg, value)
		self:UpdateStatTable()
	end 
end

function ItemScoreConfig:GetScore(info, value)
	if not self.selectedSet then
		-- display error here
		return 0
	else
		return ItemScore:GetScore(self.selectedSet, info.arg)
	end
end

function ItemScoreConfig:SetCap(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set cap value with no selected set")
	else
		if value == "" then
			value = -1
		end
		ItemScore:SetCap(self.selectedSet, info.arg, value)
		self:UpdateStatTable()
	end 
end

function ItemScoreConfig:GetCap(info, value)
	if not self.selectedSet then
		-- display error here
		return ""
	else
		local cap = ItemScore:GetCap(self.selectedSet, info.arg)
		if cap == -1 then
			return ""
		else
			return cap
		end
	end
end

function ItemScoreConfig:AddStat(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot add stat with no selected set")
	else
		ItemScore:AddStat(self.selectedSet, info.arg)
		self:UpdateStatTable()
	end 
end

function ItemScoreConfig:RemoveStat(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot remove stat with no selected set")
	else
		ItemScore:RemoveStat(self.selectedSet, info.arg)
		self:UpdateStatTable()
	end 
end

function ItemScoreConfig:GetParent(info, value)
	-- return the selected set name
	if self.selectedSet then
		return ItemScore:GetParent(self.selectedSet)
	end
	
	return nil
end

function ItemScoreConfig:SetParent(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set parent with no selected set")
	else
	    ItemScore:SetParent(self.selectedSet, value)
	end 
end

function ItemScoreConfig:SetEnabled(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set display value with no selected set")
	else
		self.selectedSet.display = value
	end 
end

function ItemScoreConfig:GetEnabled(info, value)
	if not self.selectedSet then
		-- display error here
		return nil
	else
		return ItemScore:GetEnabled(self.selectedSet)
	end
end

function ItemScoreConfig:SetPrecision(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set precision value with no selected set")
	else
		ItemScore:SetPrecision(self.selectedSet, value)
		self:UpdateExampleText()
	end 
end

function ItemScoreConfig:GetPrecision(info, value)
	if not self.selectedSet then
		-- display error here
		return 0
	else
		return ItemScore:GetPrecision(self.selectedSet)
	end
end

function ItemScoreConfig:SetColour(info, r, g, b, a)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set colour with no selected set")
	else
		ItemScore:SetColour(self.selectedSet, r, g, b)
	end 
end

function ItemScoreConfig:GetColour(info, value)
	if not self.selectedSet then
		-- display error here
		return 0,0,0,1
	else
		return ItemScore:GetColour(self.selectedSet)
	end
end

function ItemScoreConfig:SetPattern(info, value)
	if not self.selectedSet then
		DEFAULT_CHAT_FRAME:AddMessage("Cannot set text with no selected set")
	else
		ItemScore:SetPattern(self.selectedSet, value)
		self:UpdateExampleText()
	end 
end

function ItemScoreConfig:GetPattern(info, value)
	if not self.selectedSet then
		-- display error here
		return ""
	else
		return ItemScore:GetPattern(self.selectedSet)
	end
end


function ItemScoreConfig:GetImportFormat(info, value)
	if not self.importFormat then
		self.importFormat = 1
	end
	return self.importFormat
end

function ItemScoreConfig:SetImportFormat(info, value)
	self.importFormat = value
	
	-- REVISIT - enable/disable name depedning on format
end

function ItemScoreConfig:GetImportName(info, value)
	if not self.importName then
		self.importName = ""
	end
	
	return self.importName
end

function ItemScoreConfig:SetImportName(info, value)
	self.importName = value
end

function ItemScoreConfig:GetImportCode(info, value)
	if not self.importCode then
		self.importCode = ""
	end
	
	return self.importCode
end

function ItemScoreConfig:SetImportCode(info, value)
	self.importCode = value
end

function ItemScoreConfig:ImportScoreSet()
	ItemScore:ParseTag(self.importName, importFormats[self.importFormat], self.importCode)
	self:UpdateSetLists()
end

function ItemScoreConfig:UpdateExportCode()
	self.exportCode = ItemScore:ExportSet(self.exportSet, importFormats[self.exportFormat])
end

function ItemScoreConfig:GetExportSet(info, value)
	return self.exportSet
end

function ItemScoreConfig:SetExportSet(info, value)
	self.exportSet = value
	self:UpdateExportCode()
end

function ItemScoreConfig:GetExportFormat(info, value)
	if not self.exportFormat then
		self.exportFormat = "AskMrRobot"
	end
	return self.exportFormat
end

function ItemScoreConfig:SetExportFormat(info, value)
	self.exportFormat = value
	self:UpdateExportCode()
end

function ItemScoreConfig:GetExportCode(info, value)
	if not self.exportCode then
		self.exportCode = ""
	end
	
	return self.exportCode
end

